'use strict';

var stringLength = require('string-length');
var resolve = require('../resolve');
var symbols = require('../symbols');
var getFilledArray = require('../getFilledArray');

var _symbols$code = symbols.code,
    topLeft = _symbols$code.topLeft,
    topRight = _symbols$code.topRight,
    bottomLeft = _symbols$code.bottomLeft,
    bottomRight = _symbols$code.bottomRight,
    separatorTop = _symbols$code.separatorTop,
    separatorBottom = _symbols$code.separatorBottom,
    title = _symbols$code.title,
    contentLeft = _symbols$code.contentLeft,
    contentRight = _symbols$code.contentRight,
    msg = _symbols$code.msg;


var fill = function fill(length, value) {
	return getFilledArray(length, value).join('');
};

var line = function line(width, start, _content, end) {
	return '' + start + fill(width - stringLength(start) - stringLength(end), _content) + end;
};

var textRow = function textRow(_ref) {
	var row = _ref.row,
	    width = _ref.width,
	    num = _ref.num,
	    _ref$newLine = _ref.newLine,
	    newLine = _ref$newLine === undefined ? false : _ref$newLine;

	var preSymbol = contentLeft(newLine ? 0 : num);
	var postSymbol = contentRight;
	var newWidth = width - stringLength(preSymbol + '  ' + postSymbol);
	var rowLng = stringLength(row);

	if (rowLng > newWidth) {
		var out = [];
		out = out.concat(textRow({ row: '' + row.slice(0, newWidth), width: width, num: num, newLine: newLine }));
		out = out.concat(textRow({ row: '' + row.slice(newWidth, rowLng), width: width, num: num, newLine: true }));
		return out;
	}
	return ['' + preSymbol + msg(' ' + (row + fill(newWidth - rowLng, ' ')) + ' ') + postSymbol];
};

var text = function text(_ref2) {
	var rows = _ref2.rows,
	    width = _ref2.width;

	var out = [];
	rows.forEach(function (row, index) {
		var result = textRow({ row: row, width: width, num: index + 1 });
		out = out.concat(result);
	});
	return out.join('\n');
};

/**
 * @param {string} markdown - the readme files data
 * @param {Object} di - dependency injection
 * @returns {string} markdown - replaced readme file data
 **/
function convertMarkdownCodeToCli(markdown, _ref3) {
	var _process = _ref3._process;

	var reg = /(```)([a-zA-Z ]*)([^```]*)(```)/g;
	var results = void 0;
	var width = _process.stdout.columns;

	// eslint-disable-next-line
	while ((results = reg.exec(markdown)) !== null) {
		var _title = results[2].trim();
		var _text = results[3];
		var out = [''];

		// add title bar
		if (_title) {
			var topStart = '' + topLeft + title(_title);
			var topEnd = '' + topRight;
			out.push(line(width, topStart, separatorTop, topEnd));
		}

		// add empty line
		out.push(line(width, contentLeft(0), msg(' '), contentRight));

		// add content
		out.push(text({
			rows: _text.split('\n'),
			width: width
		}));

		var bottomStart = '' + bottomLeft;
		var bottomEnd = '' + bottomRight;
		out.push(line(width, bottomStart, separatorBottom, bottomEnd));

		markdown = markdown.replace(results[0], out.join('\n'));
	}

	return markdown;
}

module.exports = resolve(convertMarkdownCodeToCli, { process: process });
module.exports.convertMarkdownCodeToCli = convertMarkdownCodeToCli;
module.exports.textRow = textRow;